﻿using System;
using System.Collections;
using Android.App;
using Android.Content;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using Android.Hardware;
using Java.IO;
using Android.Media;
using Android.Graphics;

namespace SimpleCamera
{
    [Activity(Label = "SimpleCamera", MainLauncher = true)]
    public class SimpleCameraActivity : Activity,
        Android.Hardware.Camera.IAutoFocusCallback, 
        Android.Hardware.Camera.IPictureCallback, 
        Android.Hardware.Camera.IPreviewCallback,
        Android.Hardware.Camera.IShutterCallback,
        ISurfaceHolderCallback
    {
        Android.Hardware.Camera camera;
        String PICTURE_FILENAME = "picture.jpg";
        
        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);

            // Set our view from the "main" layout resource
            SetContentView(Resource.Layout.Main);

            SurfaceView surface = (SurfaceView)FindViewById(Resource.Id.Surface);
            var holder = surface.Holder;
            holder.AddCallback(this);
            holder.SetType(Android.Views.SurfaceType.PushBuffers);
            holder.SetFixedSize(300, 200);

            // Get our button from the layout resource,
            // and attach an event to it
            Button button = FindViewById<Button>(Resource.Id.MyButton);
            button.Click += delegate
            {
                if (camera != null)
                {
                    Android.Hardware.Camera.Parameters p = camera.GetParameters();
                    p.PictureFormat = (int)Format.Jpeg;
                    camera.SetParameters(p);
                    camera.TakePicture(this,this,this);
                }
                else
                {
                    Toast.MakeText(this, 
                        "There was a problem. The camera object was null.", 
                        ToastLength.Long).Show();
                }
            };

        }
        public void OnAutoFocus(bool focused, Android.Hardware.Camera camera)
        {
            if (focused)
            {
                Toast.MakeText(this, "Focused", ToastLength.Short);
            }
        }
        
        public void OnShutter() {
            
        }

        public void SurfaceCreated(ISurfaceHolder holder)
        {
            CreateCamera(holder);
        }

        private void CreateCamera(ISurfaceHolder holder)
        {
            try
            {
                if (holder != null)
                { 
                    camera = Android.Hardware.Camera.Open();
                    Android.Hardware.Camera.Parameters p = camera.GetParameters();
                    p.PictureFormat = (int) Format.Jpeg;
                    camera.SetParameters(p);
                    camera.SetPreviewCallback(this);
                    camera.Lock();
                    camera.SetPreviewDisplay(holder);
                    camera.StartPreview();
                    
                    if (PackageManager.HasSystemFeature("android.hardware.camera.autofocus"))
                    {
                        camera.AutoFocus(this);
                    }
                }
            }
            catch (System.Exception e)
            {
                Android.Util.Log.Debug("SIMPLECAMERA", e.Message);
                Console.WriteLine(e.Message);
            }
        }

        public void SurfaceDestroyed(ISurfaceHolder holder)
        {
            DestroyCamera();
            GC.Collect();
        }

        private void DestroyCamera()
        {
            if (camera != null)
            {
                camera.Unlock();
                camera.StopPreview();
                camera.SetPreviewCallback(null);
                camera.Release();
                camera = null;
            }
        }

        public void SurfaceChanged(ISurfaceHolder holder, int i, int j, int k)
        {
        }

        public void OnPreviewFrame(byte[] data, Android.Hardware.Camera camera)
        {
            //TODO: Display preview
        }

        public void OnPictureTaken(byte[] data, Android.Hardware.Camera camera)
        {
            // Save the image JPEG data to the SD card
            FileOutputStream outStream = null;
            //String dataDir = Android.OS.Environment.DataDirectory.Path;
            File dataDir = Android.OS.Environment.ExternalStorageDirectory;
                
            if (data!=null)
            {
                try
                {
                    outStream = new FileOutputStream(dataDir + "/" + PICTURE_FILENAME);
                    //outStream = new FileOutputStream("/sdcard/test.jpg");
                    outStream.Write(data);
                    outStream.Close();
                }
                catch (FileNotFoundException e)
                {
                    Android.Util.Log.Debug("SIMPLECAMERA", e.Message);
                }
                catch (IOException e)
                {
                    Android.Util.Log.Debug("SIMPLECAMERA", e.Message);
                }
    
                File file = new File(dataDir + "/" + PICTURE_FILENAME);
                try {
                    
                    ExifInterface exif = new ExifInterface(file.CanonicalPath);
                    // Read the camera model and location attributes
                    exif.GetAttribute(ExifInterface.TagModel);
                    float[] latLng = new float[2];
                    exif.GetLatLong(latLng);
                    // Set the camera make
                    exif.SetAttribute(ExifInterface.TagMake, "My Phone");
                    exif.SetAttribute(ExifInterface.TagDatetime, System.DateTime.Now.ToString());
                } 
                catch (IOException e) {
    
                    Android.Util.Log.Debug("SIMPLECAMERA", e.Message);
                }
            }
            else
            {
                Toast.MakeText(this, "No Image Captured", ToastLength.Long);
            }
        }
    }
}

